@@ -38,7 +38,8 @@ class KlippyGtk:
         self.font_size_type = screen._config.get_main_config().get("font_size", "medium")
         self.width = screen.width
         self.height = screen.height
-        self.font_ratio = [33, 49] if self.screen.vertical_mode else [43, 29]
+        self.ultra_tall = (self.height / self.width) >= 3
+        self.font_ratio = [31, 47] if self.screen.vertical_mode else [43, 29]
         self.font_size = min(self.width / self.font_ratio[0], self.height / self.font_ratio[1])
         self.img_scale = self.font_size * 2
         self.button_image_scale = 1.38
@@ -76,7 +77,7 @@ class KlippyGtk:
             self.content_height = self.height - self.titlebar_height
 
         self.keyboard_height = self.content_height * 0.5
-        if (self.height / self.width) >= 3:  # Ultra-tall
+        if self.ultra_tall:
             self.keyboard_height = self.keyboard_height * 0.5
 
         self.color_list = {}  # This is set by screen.py init_style()
@@ -122,7 +123,8 @@ class KlippyGtk:
         height = height if height is not None else self.img_height
         filename = os.path.join(self.themedir, filename)
         for ext in ["svg", "png"]:
-            pixbuf = self.PixbufFromFile(f"{filename}.{ext}", int(width), int(height))
+            file = f"{filename}.{ext}"
+            pixbuf = self.PixbufFromFile(file, int(width), int(height)) if os.path.exists(file) else None
             if pixbuf is not None:
                 return pixbuf
         return None
@@ -196,30 +198,37 @@ class KlippyGtk:
                 spinner.hide()
             widget.set_sensitive(True)
 
+    def dialog_content_decouple(self, widget, event, dialog):
+        self.remove_dialog(dialog)
+
     def Dialog(self, title, buttons, content, callback=None, *args):
         dialog = Gtk.Dialog(title=title, modal=True, transient_for=self.screen,
                             default_width=self.width, default_height=self.height)
         if not self.screen.windowed:
             dialog.fullscreen()
 
-        max_buttons = 3 if self.screen.vertical_mode else 4
-        if len(buttons) > max_buttons:
-            buttons = buttons[:max_buttons]
-        if len(buttons) > 2:
-            dialog.get_action_area().set_layout(Gtk.ButtonBoxStyle.EXPAND)
-            button_hsize = -1
-        else:
-            button_hsize = int((self.width / 3))
-        for button in buttons:
-            if 'style' in button:
-                style = button['style']
+        if buttons:
+            max_buttons = 3 if self.screen.vertical_mode else 4
+            if len(buttons) > max_buttons:
+                buttons = buttons[:max_buttons]
+            if len(buttons) > 2:
+                dialog.get_action_area().set_layout(Gtk.ButtonBoxStyle.EXPAND)
+                button_hsize = -1
             else:
-                style = 'dialog-default'
-            dialog.add_button(button['name'], button['response'])
-            button = dialog.get_widget_for_response(button['response'])
-            button.set_size_request(button_hsize, self.dialog_buttons_height)
-            button.get_style_context().add_class(style)
-            format_label(button, 2)
+                button_hsize = int((self.width / 3))
+            for button in buttons:
+                style = button['style'] if 'style' in button else 'dialog-default'
+                dialog.add_button(button['name'], button['response'])
+                button = dialog.get_widget_for_response(button['response'])
+                button.set_size_request(button_hsize, self.dialog_buttons_height)
+                button.get_style_context().add_class(style)
+                format_label(button, 2)
+        else:
+            # No buttons means clicking anywhere closes the dialog
+            content.add_events(Gdk.EventMask.BUTTON_RELEASE_MASK)
+            content.connect("button-release-event", self.dialog_content_decouple, dialog)
+            dialog.add_events(Gdk.EventMask.BUTTON_RELEASE_MASK)
+            dialog.connect("button-release-event", self.remove_dialog)
 
         dialog.connect("response", self.screen.reset_screensaver_timeout)
         dialog.connect("response", callback, *args)
@@ -255,11 +264,6 @@ class KlippyGtk:
             return
         logging.debug(f"Cannot remove dialog {dialog}")
 
-    def ToggleButton(self, text):
-        b = Gtk.ToggleButton(label=text, hexpand=True, vexpand=True)
-        b.connect("clicked", self.screen.reset_screensaver_timeout)
-        return b
-
     def ScrolledWindow(self, steppers=True):
         scroll = Gtk.ScrolledWindow(hexpand=True, vexpand=True, overlay_scrolling=False)
         scroll.add_events(Gdk.EventMask.BUTTON_PRESS_MASK |
