@@ -15,6 +15,8 @@ class Panel(ScreenPanel):
         super().__init__(screen, title)
         self.show_create = False
         self.active_mesh = None
+        section = self._printer.get_config_section("bed_mesh")
+        self.mesh_radius = section['mesh_radius'] if 'mesh_radius' in section else None
         self.profiles = {}
         self.buttons = {
             'add': self._gtk.Button("increase", " " + _("Add profile"), "color1", self.bts, Gtk.PositionType.LEFT, 1),
@@ -62,9 +64,7 @@ class Panel(ScreenPanel):
             self.profiles[self.active_mesh]['name'].get_style_context().remove_class("button_active")
         if profile == "":
             logging.info("Clearing active profile")
-            self.active_mesh = None
-            self.update_graph()
-            self.buttons['clear'].set_sensitive(False)
+            self._clear_profile()
             return
         if profile not in self.profiles:
             self.add_profile(profile)
@@ -81,22 +81,21 @@ class Panel(ScreenPanel):
         if profile is None:
             return None
         if profile == self.active_mesh:
-            bm = self._printer.get_stat("bed_mesh")
-            if bm is None:
-                logging.info(f"Unable to load active mesh: {profile}")
-                return None
-            matrix = 'probed_matrix'
+            return self._printer.get_stat("bed_mesh")
         else:
-            bm = self._printer.get_config_section(f"bed_mesh {profile}")
-            if bm is False:
-                logging.info(f"Unable to load profile: {profile}")
-                self.remove_profile(profile)
-                return None
-            matrix = 'points'
-        return bm[matrix]
+            return self._printer.get_config_section(f"bed_mesh {profile}")
 
     def update_graph(self, widget=None, profile=None):
-        self.labels['map'].update_bm(self.retrieve_bm(profile))
+        if self.ks_printer_cfg is not None:
+            invert_x = self._config.get_config()['main'].getboolean("invert_x", False)
+            invert_y = self._config.get_config()['main'].getboolean("invert_y", False)
+            self.labels['map'].set_inversion(x=invert_x, y=invert_y)
+            rotation = self.ks_printer_cfg.getint("screw_rotation", 0)
+            if rotation not in (0, 90, 180, 270):
+                rotation = 0
+            self.labels['map'].set_rotation(rotation)
+            logging.info(f"Inversion X: {invert_x} Y: {invert_y} Rotation: {rotation}")
+        self.labels['map'].update_bm(self.retrieve_bm(profile), self.mesh_radius)
         self.labels['map'].queue_draw()
 
     def add_profile(self, profile):
@@ -140,12 +139,7 @@ class Panel(ScreenPanel):
             "delete": buttons["delete"],
         }
 
-        pl = list(self.profiles)
-        if "default" in pl:
-            pl.remove('default')
-        profiles = sorted(pl)
-        pos = profiles.index(profile) + 1 if profile != "default" else 0
-
+        pos = self._get_position(profile)
         self.labels['profiles'].insert_row(pos)
         self.labels['profiles'].attach(self.profiles[profile]['row'], 0, pos, 1, 1)
         self.labels['profiles'].show_all()
@@ -187,17 +181,23 @@ class Panel(ScreenPanel):
         if profile not in self.profiles:
             return
 
+        pos = self._get_position(profile)
+        self.labels['profiles'].remove_row(pos)
+        del self.profiles[profile]
+        if not self.profiles:
+            self._clear_profile()
+
+    def _clear_profile(self):
+        self.active_mesh = None
+        self.update_graph()
+        self.buttons['clear'].set_sensitive(False)
+
+    def _get_position(self, profile):
         pl = list(self.profiles)
         if "default" in pl:
             pl.remove('default')
         profiles = sorted(pl)
-        pos = profiles.index(profile) + 1 if profile != "default" else 0
-        self.labels['profiles'].remove_row(pos)
-        del self.profiles[profile]
-        if not self.profiles:
-            self.active_mesh = None
-            self.update_graph()
-            self.buttons['clear'].set_sensitive(False)
+        return profiles.index(profile) + 1 if profile != "default" else 0
 
     def show_create_profile(self, widget):
 
