@@ -42,8 +42,6 @@ class Panel(ScreenPanel):
         self.labels['options_menu'].add(self.labels['options'])
 
     def activate(self):
-        while len(self.menu) > 1:
-            self.unload_menu()
         self.reload_macros()
 
     def add_gcode_macro(self, macro):
@@ -80,8 +78,7 @@ class Panel(ScreenPanel):
             "row": row,
             "params": {},
         }
-        pattern = r'params\.(?P<param>..*)\|default\((?P<default>..*)\).*'
-        i = 0
+        pattern = r'params\.(?P<param>[a-zA-Z0-9_]+)(?:\s*\|.*\s*default\(\s*(?P<default>[^\)]+)\))?'
         for line in gcode:
             if line.startswith("{") and "params." in line:
                 result = re.search(pattern, line)
@@ -102,7 +99,10 @@ class Panel(ScreenPanel):
         for param in self.macros[macro]["params"]:
             value = self.macros[macro]["params"][param].get_text()
             if value:
-                params += f' {param}={value}'
+                if re.findall(r'[G|M]\d{1,3}', macro):
+                    params += f' {param}{value}'
+                else:
+                    params += f' {param}={value}'
         self._screen.show_popup_message(f"{macro} {params}", 1)
         self._screen._send_action(widget, "printer.gcode.script", {"script": f"{macro}{params}"})
 
@@ -129,6 +129,7 @@ class Panel(ScreenPanel):
             self.options[macro] = {
                 "name": macro,
                 "section": f"displayed_macros {self._screen.connected_printer}",
+                "type": "binary"
             }
             show = self._config.get_config().getboolean(self.options[macro]["section"], macro.lower(), fallback=True)
             if macro not in self.macros and show:
@@ -143,36 +144,6 @@ class Panel(ScreenPanel):
             self.labels['macros'].attach(self.macros[macro]['row'], 0, pos, 1, 1)
             self.labels['macros'].show_all()
 
-    def add_option(self, boxname, opt_array, opt_name, option):
-        name = Gtk.Label(hexpand=True, vexpand=True, halign=Gtk.Align.START, valign=Gtk.Align.CENTER,
-                         wrap=True, wrap_mode=Pango.WrapMode.WORD_CHAR)
-        name.set_markup(f"<big><b>{option['name']}</b></big>")
-
-        box = Gtk.Box(vexpand=False)
-        switch = Gtk.Switch(hexpand=False, vexpand=False,
-                            width_request=round(self._gtk.font_size * 7),
-                            height_request=round(self._gtk.font_size * 3.5),
-                            active=self._config.get_config().getboolean(option['section'], opt_name, fallback=True))
-        switch.connect("notify::active", self.switch_config_option, option['section'], opt_name)
-        box.add(switch)
-
-        dev = Gtk.Box(hexpand=True, vexpand=False, valign=Gtk.Align.CENTER)
-        dev.get_style_context().add_class("frame-item")
-        dev.add(name)
-        dev.add(box)
-
-        opt_array[opt_name] = {
-            "name": option['name'],
-            "row": dev
-        }
-
-        opts = sorted(self.options, key=str.casefold)
-        pos = opts.index(opt_name)
-
-        self.labels[boxname].insert_row(pos)
-        self.labels[boxname].attach(opt_array[opt_name]['row'], 0, pos, 1, 1)
-        self.labels[boxname].show_all()
-
     def back(self):
         if len(self.menu) > 1:
             self.unload_menu()
