@@ -337,7 +337,6 @@ class Panel(ScreenPanel):
         if self.flow_timeout is not None:
             GLib.source_remove(self.flow_timeout)
             self.flow_timeout = None
-        self._files.remove_callback(self._callback_metadata)
 
     def create_buttons(self):
 
@@ -373,8 +372,15 @@ class Panel(ScreenPanel):
                             + "\n\n"
                             + _("Saved offset: %s") % saved_z_offset)
         elif device == "endstop":
-            label.set_label(_("Apply %s%.3f offset to Endstop?") % (sign, abs(self.zoffset)))
-
+            saved_z_offset = None
+            msg = _("Apply %s%.3f offset to Endstop?") % (sign, abs(self.zoffset))
+            if 'stepper_z' in self._printer.get_config_section_list():
+                saved_z_offset = self._printer.get_config_section('stepper_z')['position_endstop']
+            elif 'stepper_a' in self._printer.get_config_section_list():
+                saved_z_offset = self._printer.get_config_section('stepper_a')['position_endstop']
+            if saved_z_offset:
+                msg += "\n\n" + _("Saved offset: %s") % saved_z_offset
+            label.set_label(msg)
         buttons = [
             {"name": _("Apply"), "response": Gtk.ResponseType.APPLY, "style": 'dialog-default'},
             {"name": _("Cancel"), "response": Gtk.ResponseType.CANCEL, "style": 'dialog-error'}
@@ -447,12 +453,6 @@ class Panel(ScreenPanel):
         for arg in args:
             self.buttons[arg].set_sensitive(False)
 
-    def _callback_metadata(self, action, item):
-        if action == "update_metadata" and self.filename in item:
-            logging.info("Callback complete")
-            self.update_file_metadata()
-            self._files.remove_callback(self._callback_metadata)
-
     def new_print(self):
         self._screen.close_screensaver()
         if "virtual_sdcard" in self._printer.data:
@@ -469,6 +469,8 @@ class Panel(ScreenPanel):
             elif "action:resumed" in data:
                 self.set_state("printing")
             return
+        elif action == "notify_metadata_update" and data['filename'] == self.filename:
+            self.update_file_metadata(response=True)
         elif action != "notify_status_update":
             return
 
@@ -476,9 +478,9 @@ class Panel(ScreenPanel):
             if x in data:
                 self.update_temp(
                     x,
-                    self._printer.get_dev_stat(x, "temperature"),
-                    self._printer.get_dev_stat(x, "target"),
-                    self._printer.get_dev_stat(x, "power"),
+                    self._printer.get_stat(x, "temperature"),
+                    self._printer.get_stat(x, "target"),
+                    self._printer.get_stat(x, "power"),
                 )
                 if x in self.buttons['extruder']:
                     self.buttons['extruder'][x].set_label(self.labels[x].get_text())
@@ -593,8 +595,6 @@ class Panel(ScreenPanel):
     def update_time_left(self):
         total_duration = float(self._printer.get_stat('print_stats', 'total_duration'))
         print_duration = float(self._printer.get_stat('print_stats', 'print_duration'))
-        if not self.file_metadata.get('filament_total'):  # No-extrusion
-            print_duration = total_duration
         fila_used = float(self._printer.get_stat('print_stats', 'filament_used'))
         if "gcode_start_byte" in self.file_metadata:
             progress = (max(self._printer.get_stat('virtual_sdcard', 'file_position') -
@@ -605,6 +605,7 @@ class Panel(ScreenPanel):
         self.labels["duration"].set_label(self.format_time(total_duration))
         elapsed_label = f"{self.labels['elapsed'].get_text()}  {self.labels['duration'].get_text()}"
         self.buttons['elapsed'].set_label(elapsed_label)
+        find_widget(self.buttons['elapsed'], Gtk.Label).set_ellipsize(Pango.EllipsizeMode.END)
         estimated = slicer_time = filament_time = file_time = 0
         timeleft_type = self._config.get_config()['main'].get('print_estimate_method', 'auto')
 
@@ -612,6 +613,10 @@ class Panel(ScreenPanel):
             spdcomp = sqrt(self.speed_factor)
             slicer_time = ((self.file_metadata['estimated_time']) / spdcomp)
             self.labels["slicer_time"].set_label(self.format_time(slicer_time))
+            if print_duration < 1:
+                print_duration = slicer_time * progress
+        elif print_duration < 1:  # No-extrusion
+            print_duration = total_duration
 
         if 'filament_total' in self.file_metadata and self.file_metadata['filament_total'] >= fila_used > 0:
             filament_time = (print_duration / (fila_used / self.file_metadata['filament_total']))
@@ -627,7 +632,7 @@ class Panel(ScreenPanel):
         elif timeleft_type == "slicer":
             estimated = slicer_time
         elif estimated < 1:  # Auto
-            if slicer_time > 1:
+            if print_duration < slicer_time > 1:
                 if progress < 0.15:
                     # At the begining file and filament are innacurate
                     estimated = slicer_time
@@ -637,7 +642,7 @@ class Panel(ScreenPanel):
                 elif file_time > 1:
                     # Weighted arithmetic mean (Slicer is the most accurate)
                     estimated = (slicer_time * 2 + file_time) / 3
-            elif filament_time > 1 and file_time > 1:
+            elif print_duration < filament_time > 1 and file_time > 1:
                 estimated = (filament_time + file_time) / 2
             elif file_time > 1:
                 estimated = file_time
@@ -647,6 +652,7 @@ class Panel(ScreenPanel):
             self.labels["time_left"].set_label(self.format_eta(estimated, print_duration))
             remaining_label = f"{self.labels['left'].get_text()}  {self.labels['time_left'].get_text()}"
             self.buttons['left'].set_label(remaining_label)
+            find_widget(self.buttons['left'], Gtk.Label).set_ellipsize(Pango.EllipsizeMode.END)
         self.update_progress(progress)
 
     def update_progress(self, progress: float):
@@ -745,15 +751,12 @@ class Panel(ScreenPanel):
             image.set_from_pixbuf(pixbuf)
 
     def show_fullscreen_thumbnail(self, widget):
-        buttons = [
-            {"name": _("Close"), "response": Gtk.ResponseType.CANCEL}
-        ]
         pixbuf = self.get_file_image(self.filename, self._screen.width * .9, self._screen.height * .75)
         if pixbuf is None:
             return
         image = Gtk.Image.new_from_pixbuf(pixbuf)
         image.set_vexpand(True)
-        self._gtk.Dialog(self.filename, buttons, image, self.close_fullscreen_thumbnail)
+        self._gtk.Dialog(self.filename, None, image, self.close_fullscreen_thumbnail)
 
     def close_fullscreen_thumbnail(self, dialog, response_id):
         self._gtk.remove_dialog(dialog)
@@ -761,30 +764,32 @@ class Panel(ScreenPanel):
     def update_filename(self, filename):
         if not filename:
             return
+        if self.animation_timeout is not None:
+            GLib.source_remove(self.animation_timeout)
+            self.animation_timeout = None
         self.filename = filename
+        logging.debug(f"Updating filename to {filename}")
         self.labels["file"].set_label(os.path.splitext(self.filename)[0])
         self.filename_label = {
             "complete": self.labels['file'].get_label(),
             "current": self.labels['file'].get_label(),
-            "position": 0,
-            "limit": (self._screen.width * 37 / 480) // (self._gtk.font_size / 11),
-            "length": len(self.labels['file'].get_label())
         }
-        if self.animation_timeout is None and (self.filename_label['length'] - self.filename_label['limit']) > 0:
+        ellipsized = self.labels['file'].get_layout().is_ellipsized()
+        if ellipsized:
             self.animation_timeout = GLib.timeout_add_seconds(1, self.animate_label)
         self.update_file_metadata()
 
     def animate_label(self):
-        pos = self.filename_label['position']
-        if pos > (self.filename_label['length'] - self.filename_label['limit']):
-            self.filename_label['position'] = 0
-            self.labels['file'].set_label(self.filename_label['complete'])
+        ellipsized = self.labels['file'].get_layout().is_ellipsized()
+        if ellipsized:
+            self.filename_label['current'] = self.filename_label['current'][2:]
+            self.labels['file'].set_label(self.filename_label['current'])
         else:
-            self.labels['file'].set_label(self.filename_label['current'][pos:self.filename_label['length']])
-            self.filename_label['position'] += 1
+            self.filename_label['current'] = self.filename_label['complete']
+            self.labels['file'].set_label(self.filename_label['complete'])
         return True
 
-    def update_file_metadata(self):
+    def update_file_metadata(self, response=False):
         if self._files.file_metadata_exists(self.filename):
             self.file_metadata = self._files.get_file_info(self.filename)
             logging.info(f"Update Metadata. File: {self.filename} Size: {self.file_metadata['size']}")
@@ -802,8 +807,9 @@ class Panel(ScreenPanel):
                     self.labels['total_layers'].set_label(f"{((self.oheight - self.f_layer_h) / self.layer_h) + 1:.0f}")
             if "filament_total" in self.file_metadata:
                 self.labels['filament_total'].set_label(f"{float(self.file_metadata['filament_total']) / 1000:.1f} m")
-        else:
+        elif not response:
             logging.debug("Cannot find file metadata. Listening for updated metadata")
-            self._screen.files.add_callback(self._callback_metadata)
             self._files.request_metadata(self.filename)
+        else:
+            logging.debug("Cannot load file metadata")
         self.show_file_thumbnail()
