@@ -25,24 +25,22 @@ class Panel(MenuPanel):
         stats = self._printer.get_printer_status_data()["printer"]
         if stats["temperature_devices"]["count"] > 0 or stats["extruders"]["count"] > 0:
             self._gtk.reset_temp_color()
-            self.main_menu.attach(self.create_left_panel(), 0, 0, 1, 1)
         if self._screen.vertical_mode:
+            self.main_menu.attach(self.create_left_panel(), 0, 0, 1, 3)
             self.labels['menu'] = self.arrangeMenuItems(items, 3, True)
             scroll.add(self.labels['menu'])
-            self.main_menu.attach(scroll, 0, 1, 1, 1)
+            self.main_menu.attach(scroll, 0, 3, 1, 2)
         else:
+            self.main_menu.attach(self.create_left_panel(), 0, 0, 1, 1)
             self.labels['menu'] = self.arrangeMenuItems(items, 2, True)
             scroll.add(self.labels['menu'])
             self.main_menu.attach(scroll, 1, 0, 1, 1)
         self.content.add(self.main_menu)
 
-    def update_graph_visibility(self):
+    def update_graph_visibility(self, force_hide=False):
         if self.left_panel is None:
             logging.info("No left panel")
             return
-        if not self._printer.get_temp_store_devices():
-            logging.debug(f"Could not create graph tempstore: {self._printer.get_temp_store_devices()}")
-            return
         count = 0
         for device in self.devices:
             visible = self._config.get_config().getboolean(f"graph {self._screen.connected_printer}",
@@ -54,7 +52,7 @@ class Panel(MenuPanel):
                 self.devices[device]['name'].get_style_context().add_class("graph_label")
             else:
                 self.devices[device]['name'].get_style_context().remove_class("graph_label")
-        if count > 0:
+        if count > 0 and not force_hide:
             if self.labels['da'] not in self.left_panel:
                 self.left_panel.add(self.labels['da'])
             self.labels['da'].queue_draw()
@@ -70,7 +68,7 @@ class Panel(MenuPanel):
         return False
 
     def activate(self):
-        if self._printer.tempstore is None:
+        if not self._printer.tempstore:
             self._screen.init_tempstore()
         self.update_graph_visibility()
 
@@ -85,7 +83,7 @@ class Panel(MenuPanel):
 
         logging.info(f"Adding device: {device}")
 
-        temperature = self._printer.get_dev_stat(device, "temperature")
+        temperature = self._printer.get_stat(device, "temperature")
         if temperature is None:
             return False
 
@@ -127,9 +125,11 @@ class Panel(MenuPanel):
         rgb = self._gtk.get_temp_color(dev_type)
 
         can_target = self._printer.device_has_target(device)
-        self.labels['da'].add_object(device, "temperatures", rgb, False, True)
+        self.labels['da'].add_object(device, "temperatures", rgb, False, False)
         if can_target:
-            self.labels['da'].add_object(device, "targets", rgb, True, False)
+            self.labels['da'].add_object(device, "targets", rgb, False, True)
+        if self._show_heater_power and self._printer.device_has_power(device):
+            self.labels['da'].add_object(device, "powers", rgb, True, False)
 
         name = self._gtk.Button(image, self.prettify(devname), None, self.bts, Gtk.PositionType.LEFT, 1)
         name.connect("clicked", self.toggle_visibility, device)
@@ -190,7 +190,6 @@ class Panel(MenuPanel):
         else:
             logging.info(f"Unknown heater: {self.active_heater}")
             self._screen.show_popup_message(_("Unknown Heater") + " " + self.active_heater)
-        self._printer.set_dev_stat(self.active_heater, "target", temp)
 
     def verify_max_temp(self, temp):
         temp = int(temp)
@@ -224,7 +223,7 @@ class Panel(MenuPanel):
         self.labels['devices'].attach(name, 0, 0, 1, 1)
         self.labels['devices'].attach(temp, 1, 0, 1, 1)
 
-        self.labels['da'] = HeaterGraph(self._printer, self._gtk.font_size)
+        self.labels['da'] = HeaterGraph(self._screen, self._printer, self._gtk.font_size)
 
         scroll = self._gtk.ScrolledWindow(steppers=False)
         scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
@@ -244,11 +243,17 @@ class Panel(MenuPanel):
         self.active_heater = None
 
         if self._screen.vertical_mode:
-            self.main_menu.remove_row(1)
-            self.main_menu.attach(self.labels['menu'], 0, 1, 1, 1)
+            if not self._gtk.ultra_tall:
+                self.update_graph_visibility(force_hide=False)
+            top = self.main_menu.get_child_at(0, 0)
+            bottom = self.main_menu.get_child_at(0, 2)
+            self.main_menu.remove(top)
+            self.main_menu.remove(bottom)
+            self.main_menu.attach(top, 0, 0, 1, 3)
+            self.main_menu.attach(self.labels["menu"], 0, 3, 1, 2)
         else:
             self.main_menu.remove_column(1)
-            self.main_menu.attach(self.labels['menu'], 1, 0, 1, 1)
+            self.main_menu.attach(self.labels["menu"], 1, 0, 1, 1)
         self.main_menu.show_all()
         self.numpad_visible = False
         self._screen.base_panel.set_control_sensitive(False, control='back')
@@ -260,9 +265,9 @@ class Panel(MenuPanel):
             if x in data:
                 self.update_temp(
                     x,
-                    self._printer.get_dev_stat(x, "temperature"),
-                    self._printer.get_dev_stat(x, "target"),
-                    self._printer.get_dev_stat(x, "power"),
+                    self._printer.get_stat(x, "temperature"),
+                    self._printer.get_stat(x, "target"),
+                    self._printer.get_stat(x, "power"),
                 )
 
     def show_numpad(self, widget, device):
@@ -280,8 +285,14 @@ class Panel(MenuPanel):
         self.labels["keypad"].clear()
 
         if self._screen.vertical_mode:
-            self.main_menu.remove_row(1)
-            self.main_menu.attach(self.labels["keypad"], 0, 1, 1, 1)
+            if not self._gtk.ultra_tall:
+                self.update_graph_visibility(force_hide=True)
+            top = self.main_menu.get_child_at(0, 0)
+            bottom = self.main_menu.get_child_at(0, 3)
+            self.main_menu.remove(top)
+            self.main_menu.remove(bottom)
+            self.main_menu.attach(top, 0, 0, 1, 2)
+            self.main_menu.attach(self.labels["keypad"], 0, 2, 1, 2)
         else:
             self.main_menu.remove_column(1)
             self.main_menu.attach(self.labels["keypad"], 1, 0, 1, 1)
