@@ -12,8 +12,7 @@ class Panel(ScreenPanel):
         self.devices = {}
 
         # Create a grid for all devices
-        self.labels['devices'] = Gtk.Grid()
-        self.labels['devices'].set_valign(Gtk.Align.CENTER)
+        self.labels['devices'] = Gtk.Grid(valign=Gtk.Align.CENTER)
 
         self.load_power_devices()
 
@@ -32,29 +31,20 @@ class Panel(ScreenPanel):
             self.devices[x]['switch'].connect("notify::active", self.on_switch, x)
 
     def add_device(self, device):
-        name = Gtk.Label()
+        name = Gtk.Label(
+            hexpand=True, vexpand=True, halign=Gtk.Align.START, valign=Gtk.Align.CENTER,
+            wrap=True, wrap_mode=Pango.WrapMode.WORD_CHAR)
         name.set_markup(f"<big><b>{device}</b></big>")
-        name.set_hexpand(True)
-        name.set_vexpand(True)
-        name.set_halign(Gtk.Align.START)
-        name.set_valign(Gtk.Align.CENTER)
-        name.set_line_wrap(True)
-        name.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
-
-        switch = Gtk.Switch()
-        switch.set_hexpand(False)
-        switch.set_active(self._printer.get_power_device_status(device) == "on")
+        switch = Gtk.Switch(hexpand=False, active=(self._printer.get_power_device_status(device) == "on"),
+                            width_request=round(self._gtk.font_size * 7),
+                            height_request=round(self._gtk.font_size * 3.5))
         switch.connect("notify::active", self.on_switch, device)
-        switch.set_property("width-request", round(self._gtk.font_size * 7))
-        switch.set_property("height-request", round(self._gtk.font_size * 3.5))
 
         labels = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         labels.add(name)
 
-        dev = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
-        dev.set_hexpand(True)
-        dev.set_vexpand(False)
-        dev.set_valign(Gtk.Align.CENTER)
+        dev = Gtk.Box(
+            spacing=5, hexpand=True, vexpand=False, valign=Gtk.Align.CENTER)
         dev.add(labels)
         dev.add(switch)
 
