@@ -1,5 +1,4 @@
 import logging
-import re
 import gi
 
 gi.require_version("Gtk", "3.0")
@@ -68,20 +67,6 @@ class Panel(ScreenPanel):
             for opt in self.list:
                 if opt in data["firmware_retraction"]:
                     self.update_option(opt, data["firmware_retraction"][opt])
-        elif action == "notify_gcode_response":
-            # // RETRACT_LENGTH=0.00000 RETRACT_SPEED=20.00000 UNRETRACT_EXTRA_LENGTH=0.00000 UNRETRACT_SPEED=10.00000
-            result = re.match(
-                "^// [RETRACT_LENGTH= ]+([\\-0-9\\.]+)" +
-                "[RETRACT_SPEED= ]+([\\-0-9\\.]+)" +
-                "[UNRETRACT_EXTRA_LENGTH= ]+([\\-0-9\\.]+)" +
-                "[UNRETRACT_SPEED= ]+([\\-0-9\\.]+)",
-                data
-            )
-            if result:
-                self.update_option('retract_length', result[1])
-                self.update_option('retract_speed', result[2])
-                self.update_option('unretract_extra_length', result[3])
-                self.update_option('unretract_speed', result[4])
 
     def update_option(self, option, value):
         if option not in self.list:
@@ -106,22 +91,16 @@ class Panel(ScreenPanel):
     def add_option(self, option, optname, units, value, digits, maxval):
         logging.info(f"Adding option: {option}")
 
-        name = Gtk.Label()
+        name = Gtk.Label(
+            hexpand=True, vexpand=True, halign=Gtk.Align.START, valign=Gtk.Align.CENTER,
+            wrap=True, wrap_mode=Pango.WrapMode.WORD_CHAR)
         name.set_markup(f"<big><b>{optname}</b></big> ({units})")
-        name.set_hexpand(True)
-        name.set_vexpand(True)
-        name.set_halign(Gtk.Align.START)
-        name.set_valign(Gtk.Align.CENTER)
-        name.set_line_wrap(True)
-        name.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
         minimum = 1 if option in ["retract_speed", "unretract_speed"] else 0
         self.values[option] = value
         # adj (value, lower, upper, step_increment, page_increment, page_size)
         adj = Gtk.Adjustment(value, minimum, maxval, 1, 5, 0)
-        scale = Gtk.Scale.new(orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj)
-        scale.set_digits(digits)
-        scale.set_hexpand(True)
-        scale.set_has_origin(True)
+        scale = Gtk.Scale(adjustment=adj, digits=digits, hexpand=True,
+                          has_origin=True)
         scale.get_style_context().add_class("option_slider")
         scale.connect("button-release-event", self.set_opt_value, option)
 
