@@ -32,26 +32,6 @@ from panels.base_panel import BasePanel
 
 logging.getLogger("urllib3").setLevel(logging.WARNING)
 
-PRINTER_BASE_STATUS_OBJECTS = [
-    'bed_mesh',
-    'configfile',
-    'display_status',
-    'extruder',
-    'fan',
-    'gcode_move',
-    'heater_bed',
-    'idle_timeout',
-    'pause_resume',
-    'print_stats',
-    'toolhead',
-    'virtual_sdcard',
-    'webhooks',
-    'motion_report',
-    'firmware_retraction',
-    'exclude_object',
-    'manual_probe',
-]
-
 klipperscreendir = pathlib.Path(__file__).parent.resolve()
 
 
@@ -70,13 +50,7 @@ def set_text_direction(lang=None):
     return True
 
 
-def state_execute(callback):
-    callback()
-    return False
-
-
 class KlipperScreen(Gtk.Window):
-    """ Class for creating a screen for Klipper via HDMI """
     _cur_panels = []
     connecting = False
     connecting_to_printer = None
@@ -100,6 +74,7 @@ class KlipperScreen(Gtk.Window):
     prompt = None
 
     def __init__(self, args):
+        self.server_info = None
         try:
             super().__init__(title="KlipperScreen")
         except Exception as e:
@@ -123,7 +98,8 @@ class KlipperScreen(Gtk.Window):
 
         self.connect("key-press-event", self._key_press_event)
         self.connect("configure_event", self.update_size)
-        monitor_amount = Gdk.Display.get_n_monitors(Gdk.Display.get_default())
+        display = Gdk.Display.get_default()
+        monitor_amount = Gdk.Display.get_n_monitors(display)
         try:
             mon_n = int(args.monitor)
             if not (-1 < mon_n < monitor_amount):
@@ -131,8 +107,9 @@ class KlipperScreen(Gtk.Window):
         except ValueError:
             mon_n = 0
         logging.info(f"Monitors: {monitor_amount} using number: {mon_n}")
-        monitor = Gdk.Display.get_default().get_monitor(mon_n)
-        self.wayland = Gdk.Display.get_default().get_primary_monitor() is None
+        monitor = display.get_monitor(mon_n)
+        self.wayland = display.get_name().startswith('wayland') or display.get_primary_monitor() is None
+        logging.info(f"Wayland: {self.wayland} Display name: {display.get_name()}")
         self.width = self._config.get_main_config().getint("width", None)
         self.height = self._config.get_main_config().getint("height", None)
         if 'XDG_CURRENT_DESKTOP' in os.environ:
@@ -186,6 +163,15 @@ class KlipperScreen(Gtk.Window):
                                      + _("Ended official support in June 2023") + "\n"
                                      + _("KlipperScreen will drop support in June 2024"), 2)
 
+    def state_execute(self, state, callback):
+        self.close_screensaver()
+        if state in ("printing", "paused"):
+            self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking_printing'))
+        else:
+            self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
+        callback()
+        return False
+
     def initial_connection(self):
         self.printers = self._config.get_printers()
         state_callbacks = {
@@ -198,7 +184,7 @@ class KlipperScreen(Gtk.Window):
             "shutdown": self.state_shutdown
         }
         for printer in self.printers:
-            printer["data"] = Printer(state_execute, state_callbacks)
+            printer["data"] = Printer(self.state_execute, state_callbacks)
         default_printer = self._config.get_main_config().get('default_printer')
         logging.debug(f"Default printer: {default_printer}")
         if [True for p in self.printers if default_printer in p]:
@@ -210,18 +196,21 @@ class KlipperScreen(Gtk.Window):
             self.base_panel.show_printer_select(True)
             self.show_printer_select()
 
+    def close_websocket(self):
+        self._ws.close()
+        self.connected_printer = None
+        self.printer.state = "disconnected"
+
     def connect_printer(self, name):
         self.connecting_to_printer = name
-        if self.files:
-            self.files.__init__(self)
-        gc.collect()
         if self._ws is not None and self._ws.connected:
-            self._ws.close()
-            self.connected_printer = None
-            self.printer.state = "disconnected"
+            self.printer_initializing("Waiting Websocket closure")
+            self.close_websocket()
+            return
+        gc.collect()
         self.connecting = True
         self.initialized = False
-
+        self.initializing = False
         logging.info(f"Connecting to printer: {name}")
         ind = next(
             (
@@ -237,21 +226,25 @@ class KlipperScreen(Gtk.Window):
             self.printers[ind][name]["moonraker_port"],
             self.printers[ind][name]["moonraker_api_key"],
         )
-
-        self.printer_initializing(_("Connecting to %s") % name, remove=True)
-
-        self._ws = KlippyWebsocket(self,
-                                   {
-                                       "on_connect": self.init_printer,
-                                       "on_message": self._websocket_callback,
-                                       "on_close": self.websocket_disconnected
-                                   },
-                                   self.printers[ind][name]["moonraker_host"],
-                                   self.printers[ind][name]["moonraker_port"],
-                                   )
+        self._ws = KlippyWebsocket(
+            {
+                "on_connect": self.websocket_connected,
+                "on_message": self._websocket_callback,
+                "on_close": self.websocket_disconnected,
+                "on_cancel": self.websocket_connection_cancel,
+            },
+            self.printers[ind][name]["moonraker_host"],
+            self.printers[ind][name]["moonraker_port"],
+            self.printers[ind][name]["moonraker_api_key"],
+        )
         if self.files is None:
             self.files = KlippyFiles(self)
-        self._ws.initial_connect()
+        else:
+            self.files.reinit()
+
+        self.reinit_count = 0
+        self.printer_initializing(_("Connecting to %s") % name, True)
+        self.connect_to_moonraker()
 
     def ws_subscribe(self):
         requested_updates = {
@@ -266,7 +259,7 @@ class KlipperScreen(Gtk.Window):
                 "print_stats": ["print_duration", "total_duration", "filament_used", "filename", "state", "message",
                                 "info"],
                 "toolhead": ["homed_axes", "estimated_print_time", "print_time", "position", "extruder",
-                             "max_accel", "max_accel_to_decel", "max_velocity", "square_corner_velocity"],
+                             "max_accel", "minimum_cruise_ratio", "max_velocity", "square_corner_velocity"],
                 "virtual_sdcard": ["file_position", "is_active", "progress"],
                 "webhooks": ["state", "state_message"],
                 "firmware_retraction": ["retract_length", "retract_speed", "unretract_extra_length", "unretract_speed"],
@@ -289,7 +282,7 @@ class KlipperScreen(Gtk.Window):
             requested_updates['objects'][f] = ["speed"]
         for f in self.printer.get_filament_sensors():
             requested_updates['objects'][f] = ["enabled", "filament_detected"]
-        for p in self.printer.get_output_pins():
+        for p in self.printer.get_pwm_tools() + self.printer.get_output_pins():
             requested_updates['objects'][p] = ["value"]
         for led in self.printer.get_leds():
             requested_updates['objects'][led] = ["color_data"]
@@ -332,6 +325,8 @@ class KlipperScreen(Gtk.Window):
     def attach_panel(self, panel):
         self.base_panel.add_content(self.panels[panel])
         logging.debug(f"Current panel hierarchy: {' > '.join(self._cur_panels)}")
+        while len(self.panels[panel].menu) > 1:
+            self.panels[panel].unload_menu()
         if hasattr(self.panels[panel], "process_update"):
             self.process_update("notify_status_update", self.printer.data)
         if hasattr(self.panels[panel], "activate"):
@@ -409,8 +404,8 @@ class KlipperScreen(Gtk.Window):
         version = Gtk.Label(label=f"{functions.get_software_version()}", halign=Gtk.Align.END)
 
         help_msg = _("Provide KlipperScreen.log when asking for help.\n")
-        message = Gtk.Label(label=f"{help_msg}\n\n{e}", wrap=True)
-        scroll = self.gtk.ScrolledWindow(steppers=False)
+        message = Gtk.Label(label=f"{help_msg}\n\n{e}", wrap=True, wrap_mode=Pango.WrapMode.CHAR)
+        scroll = self.gtk.ScrolledWindow()
         scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scroll.add(message)
 
@@ -551,6 +546,8 @@ class KlipperScreen(Gtk.Window):
         logging.debug("Showing Screensaver")
         if self.screensaver is not None:
             self.close_screensaver()
+        if self.blanking_time == 0:
+            return False
         self.remove_keyboard()
         self.close_popup_message()
         for dialog in self.dialogs:
@@ -571,9 +568,6 @@ class KlipperScreen(Gtk.Window):
         self.screensaver = box
         self.screensaver.show_all()
         self.power_devices(None, self._config.get_main_config().get("screen_off_devices", ""), on=False)
-        if self.screensaver_timeout is not None:
-            GLib.source_remove(self.screensaver_timeout)
-            self.screensaver_timeout = None
         return False
 
     def close_screensaver(self, widget=None):
@@ -616,7 +610,14 @@ class KlipperScreen(Gtk.Window):
     def set_dpms(self, use_dpms):
         self.use_dpms = use_dpms
         logging.info(f"DPMS set to: {self.use_dpms}")
-        self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
+        if self.printer.state in ("printing", "paused"):
+            self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking_printing'))
+        else:
+            self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
+
+    def set_screenblanking_printing_timeout(self, time):
+        if self.printer.state in ("printing", "paused"):
+            self.set_screenblanking_timeout(time)
 
     def set_screenblanking_timeout(self, time):
         if not self.wayland:
@@ -625,9 +626,7 @@ class KlipperScreen(Gtk.Window):
 
         if time == "off":
             logging.debug(f"Screen blanking: {time}")
-            if self.screensaver_timeout is not None:
-                GLib.source_remove(self.screensaver_timeout)
-                self.screensaver_timeout = None
+            self.blanking_time = 0
             if not self.wayland:
                 os.system("xset -display :0 dpms 0 0 0")
             return
@@ -659,32 +658,49 @@ class KlipperScreen(Gtk.Window):
         self.base_panel.show_heaters(False)
         self.show_panel("printer_select", _("Printer Select"), remove_all=True)
 
-    def websocket_disconnected(self, msg):
+    def websocket_connection_cancel(self):
+        self.printer_initializing(
+            _("Cannot connect to Moonraker") + '\n\n'
+            + f'{self.apiclient.status}'
+        )
+
+    def websocket_connected(self):
+        logging.debug("### websocket_connected")
+        self._ws.klippy.identify_client(functions.get_software_version(), self._ws.api_key)
+        self.reinit_count = 0
+        self.connecting = False
+        self.connected_printer = self.connecting_to_printer
+        self.base_panel.set_ks_printer_cfg(self.connected_printer)
+        self.init_moonraker_components()
+        self.init_klipper()
+
+    def websocket_disconnected(self):
         logging.debug("### websocket_disconnected")
-        self.printer_initializing(msg, remove=True)
         self.printer.state = "disconnected"
         self.connecting = True
         self.connected_printer = None
         self.initialized = False
-        self.connect_printer(self.connecting_to_printer)
+        if 'printer_select' not in self._cur_panels:
+            self.printer_initializing(_("Lost Connection to Moonraker"), go_to_splash=True)
+            self.connect_printer(self.connecting_to_printer)
+        else:
+            self.panels['printer_select'].disconnected_callback()
 
     def state_disconnected(self):
         logging.debug("### Going to disconnected")
         self.printer.stop_tempstore_updates()
-        self.close_screensaver()
         self.initialized = False
         self.reinit_count = 0
-        self._init_printer(_("Klipper has disconnected"), remove=True)
+        self._init_printer(_("Klipper has disconnected"), go_to_splash=True)
 
     def state_error(self):
-        self.close_screensaver()
         msg = _("Klipper has encountered an error.") + "\n"
         state = self.printer.get_stat("webhooks", "state_message")
         if "FIRMWARE_RESTART" in state:
             msg += _("A FIRMWARE_RESTART may fix the issue.") + "\n"
         elif "micro-controller" in state:
             msg += _("Please recompile and flash the micro-controller.") + "\n"
-        self.printer_initializing(msg + "\n" + state, remove=True)
+        self.printer_initializing(msg + "\n" + state, go_to_splash=True)
 
     def state_paused(self):
         self.state_printing()
@@ -692,7 +708,6 @@ class KlipperScreen(Gtk.Window):
             self.show_panel("extrude", _("Extrude"))
 
     def state_printing(self):
-        self.close_screensaver()
         for dialog in self.dialogs:
             self.gtk.remove_dialog(dialog)
         self.show_panel("job_status", _("Printing"), remove_all=True)
@@ -705,17 +720,17 @@ class KlipperScreen(Gtk.Window):
             logging.debug("Printer not initialized yet")
             self.printer.state = "not ready"
             return
+        self.files.refresh_files()
         self.show_panel("main_menu", None, remove_all=True, items=self._config.get_menu_items("__main"))
 
     def state_startup(self):
         self.printer_initializing(_("Klipper is attempting to start"))
 
     def state_shutdown(self):
-        self.close_screensaver()
         self.printer.stop_tempstore_updates()
         msg = self.printer.get_stat("webhooks", "state_message")
         msg = msg if "ready" not in msg else ""
-        self.printer_initializing(_("Klipper has shutdown") + "\n\n" + msg, remove=True)
+        self.printer_initializing(_("Klipper has shutdown") + "\n\n" + msg, go_to_splash=True)
 
     def toggle_shortcut(self, show):
         if show and not self.printer.get_printer_status_data()["printer"]["gcode_macros"]["count"] > 0:
@@ -746,6 +761,7 @@ class KlipperScreen(Gtk.Window):
 
     def _websocket_callback(self, action, data):
         if self.connecting:
+            logging.debug("Not connected")
             return
         if action == "notify_klippy_disconnected":
             self.printer.process_update({'webhooks': {'state': "disconnected"}})
@@ -756,7 +772,7 @@ class KlipperScreen(Gtk.Window):
         elif action == "notify_klippy_ready":
             if not self.initialized:
                 self.reinit_count = 0
-                self._init_printer("Reconnecting", klipper=True)
+                self.init_klipper()
                 return
             self.printer.process_update({'webhooks': {'state': "ready"}})
             return
@@ -856,8 +872,8 @@ class KlipperScreen(Gtk.Window):
             if isinstance(x, Gtk.Button):
                 GLib.timeout_add(150, self.gtk.Button_busy, x, False)
 
-    def printer_initializing(self, msg, remove=False):
-        if 'splash_screen' not in self.panels or remove:
+    def printer_initializing(self, msg, go_to_splash=False):
+        if 'splash_screen' not in self.panels or go_to_splash:
             self.show_panel("splash_screen", None, remove_all=True)
         self.panels['splash_screen'].update_text(msg)
         self.log_notification(msg, 0)
@@ -881,118 +897,142 @@ class KlipperScreen(Gtk.Window):
             else:
                 self._ws.klippy.power_device_off(dev)
 
-    def _init_printer(self, msg, remove=False, klipper=False):
-        self.printer_initializing(msg, remove)
+    def _init_printer(self, msg, go_to_splash=False):
+        self.printer_initializing(msg, go_to_splash)
         self.initializing = False
-        if klipper:
-            GLib.timeout_add_seconds(3, self.init_klipper)
+        if self._ws.connected and not self._ws.closing:
+            GLib.timeout_add_seconds(4, self.init_klipper)
         else:
-            GLib.timeout_add_seconds(3, self.init_printer)
+            GLib.timeout_add_seconds(4, self.connect_to_moonraker)
 
-    def init_printer(self):
+    def connect_to_moonraker(self):
         if self.initializing:
             logging.info("Already Initializing")
             return False
+        if self._ws.closing:
+            logging.info("Cancelling attempt")
+            return False
         self.initializing = True
         if self.reinit_count > self.max_retries or 'printer_select' in self._cur_panels:
             logging.info("Stopping Retries")
+            self.printer_initializing(_("Cannot connect to Moonraker") + "\n\n"
+                                      + f'{self.apiclient.status}')
             self.initializing = False
             return False
-        state = self.apiclient.get_server_info()
-        if state is False:
-            logging.info("Moonraker not connected")
+        self.server_info = self.apiclient.get_server_info()
+        if not self.server_info:
+            logging.info("Cannot get server info")
+            if self.reinit_count > 0:
+                self._init_printer(_("Cannot connect to Moonraker") + "\n\n"
+                                   + _("Retrying") + f" #{self.reinit_count}")
+            else:
+                self._init_printer(_("Connecting to %s") % self.connecting_to_printer)
             self.initializing = False
+            self.reinit_count += 1
             return False
-        self.connecting = not self._ws.connected
-        self.connected_printer = self.connecting_to_printer
-        self.base_panel.set_ks_printer_cfg(self.connected_printer)
+        self._ws.initial_connect()
+        return False
 
-        self.init_server(state["result"])
-        # Moonraker is ready, set a loop to init the printer
-        return self.init_klipper(state["result"])
-
-    def init_server(self, server_info):
+    def init_moonraker_components(self):
         popup = ''
         level = 2
-        if server_info["warnings"]:
+        if self.server_info["warnings"]:
             popup += '\nMoonraker warnings:\n'
-            for warning in server_info["warnings"]:
+            for warning in self.server_info["warnings"]:
                 warning = warning.replace('<br>', '').replace('<br/>', '\n').replace('</br>', '\n').replace(':', ':\n')
                 popup += f"{warning}\n"
-        if server_info["failed_components"]:
+        if self.server_info["failed_components"]:
             popup += '\nMoonraker failed components:\n'
-            for failed in server_info["failed_components"]:
+            for failed in self.server_info["failed_components"]:
                 popup += f'[{failed}]\n'
-        if server_info["missing_klippy_requirements"]:
+        if self.server_info["missing_klippy_requirements"]:
             popup += '\nMissing Klipper configuration:\n'
-            for missing in server_info["missing_klippy_requirements"]:
+            for missing in self.server_info["missing_klippy_requirements"]:
                 popup += f'[{missing}]\n'
                 level = 3
         if popup:
             self.show_popup_message(popup, level)
-        if "power" in server_info["components"]:
+        if "power" in self.server_info["components"]:
             powerdevs = self.apiclient.send_request("machine/device_power/devices")
             if powerdevs is not False:
-                self.printer.configure_power_devices(powerdevs['result'])
-        if "webcam" in server_info["components"]:
+                self.printer.configure_power_devices(powerdevs)
+        if "webcam" in self.server_info["components"]:
             cameras = self.apiclient.send_request("server/webcams/list")
             if cameras is not False:
-                self.printer.configure_cameras(cameras['result']['webcams'])
-        if "spoolman" in server_info["components"]:
+                self.printer.configure_cameras(cameras['webcams'])
+        if "spoolman" in self.server_info["components"]:
             self.printer.enable_spoolman()
 
-    def init_klipper(self, server_info=None):
+    def init_klipper(self):
         if self.reinit_count > self.max_retries or 'printer_select' in self._cur_panels:
             logging.info("Stopping Retries")
             return False
-        if not server_info:
-            server_info = self.apiclient.get_server_info()["result"]
-        logging.info(f"Moonraker info {server_info}")
-
         self.reinit_count += 1
-
-        if server_info['klippy_connected'] is False:
-            logging.info("Klipper not connected")
+        self.server_info = self.apiclient.get_server_info()
+        logging.info(f"Moonraker info {self.server_info}")
+        if self.server_info['klippy_connected'] is False:
             msg = _("Moonraker: connected") + "\n\n"
-            msg += f"Klipper: {server_info['klippy_state']}" + "\n\n"
+            msg += f"Klipper: {self.server_info['klippy_state']}" + "\n\n"
             if self.reinit_count <= self.max_retries:
                 msg += _("Retrying") + f' #{self.reinit_count}'
-            return self._init_printer(msg, klipper=True)
+            self.printer_initializing(msg)
+            GLib.timeout_add_seconds(3, self.init_klipper)
+            return False
         printer_info = self.apiclient.get_printer_info()
         if printer_info is False:
-            return self._init_printer("Unable to get printer info from moonraker")
+            self._init_printer("Unable to get printer info from moonraker")
+            return False
         config = self.apiclient.send_request("printer/objects/query?configfile")
         if config is False:
-            return self._init_printer("Error getting printer configuration")
-        logging.debug(config['result']['status'])
-        # Reinitialize printer, in case the printer was shut down and anything has changed.
-        self.printer.reinit(printer_info['result'], config['result']['status'])
-        self.printer.available_commands = self.apiclient.get_gcode_help()['result']
+            self._init_printer("Error getting printer configuration")
+            return False
+        self.printer.reinit(printer_info, config['status'])
+        self.printer.available_commands = self.apiclient.get_gcode_help()
+        info = self.apiclient.send_request("machine/system_info")
+        if info and 'system_info' in info:
+            self.printer.system_info = info['system_info']
 
-        self.ws_subscribe()
-        extra_items = (self.printer.get_tools()
-                       + self.printer.get_heaters()
-                       + self.printer.get_temp_sensors()
-                       + self.printer.get_fans()
-                       + self.printer.get_temp_fans()
-                       + self.printer.get_filament_sensors()
-                       + self.printer.get_output_pins()
-                       + self.printer.get_leds()
-                       )
+        items = (
+            'bed_mesh',
+            'configfile',
+            'display_status',
+            'extruder',
+            'fan',
+            'gcode_move',
+            'heater_bed',
+            'idle_timeout',
+            'pause_resume',
+            'print_stats',
+            'toolhead',
+            'virtual_sdcard',
+            'webhooks',
+            'motion_report',
+            'firmware_retraction',
+            'exclude_object',
+            'manual_probe',
+            *self.printer.get_tools(),
+            *self.printer.get_heaters(),
+            *self.printer.get_temp_sensors(),
+            *self.printer.get_fans(),
+            *self.printer.get_temp_fans(),
+            *self.printer.get_filament_sensors(),
+            *self.printer.get_output_pins(),
+            *self.printer.get_leds(),
+        )
 
-        data = self.apiclient.send_request("printer/objects/query?" + "&".join(PRINTER_BASE_STATUS_OBJECTS +
-                                                                               extra_items))
+        data = self.apiclient.send_request("printer/objects/query?" + "&".join(items))
         if data is False:
-            return self._init_printer("Error getting printer object data with extra items")
+            self._init_printer("Error getting printer object data")
+            return False
+        self.ws_subscribe()
 
         self.files.set_gcodes_path()
-        self.files.refresh_files()
 
         logging.info("Printer initialized")
         self.initialized = True
         self.reinit_count = 0
         self.initializing = False
-        self.printer.process_update(data['result']['status'])
+        self.printer.process_update(data['status'])
         self.log_notification("Printer Initialized", 1)
         return False
 
@@ -1000,18 +1040,21 @@ class KlipperScreen(Gtk.Window):
         if len(self.printer.get_temp_devices()) == 0:
             return
         tempstore = self.apiclient.send_request("server/temperature_store")
-        if tempstore and 'result' in tempstore and tempstore['result']:
-            self.printer.init_temp_store(tempstore['result'])
+        if tempstore:
+            self.printer.init_temp_store(tempstore)
             if hasattr(self.panels[self._cur_panels[-1]], "update_graph_visibility"):
                 self.panels[self._cur_panels[-1]].update_graph_visibility()
         else:
             logging.error(f'Tempstore not ready: {tempstore} Retrying in 5 seconds')
             GLib.timeout_add_seconds(5, self.init_tempstore)
             return
+        if set(self.printer.tempstore) != set(self.printer.get_temp_devices()):
+            GLib.timeout_add_seconds(5, self.init_tempstore)
+            return
         server_config = self.apiclient.send_request("server/config")
         if server_config:
             try:
-                self.printer.tempstore_size = server_config["result"]["config"]["data_store"]["temperature_store_size"]
+                self.printer.tempstore_size = server_config["config"]["data_store"]["temperature_store_size"]
                 logging.info(f"Temperature store size: {self.printer.tempstore_size}")
             except KeyError:
                 logging.error("Couldn't get the temperature store size")
